@@ -1,8 +1,7 @@
 /* functions to deal with CGI parameters
  *
- * ----START-LICENCE----
  * Copyright 1999,2000,2001 BrightStation PLC
- * Copyright 2002 Olly Betts
+ * Copyright 2002,2007 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,11 +15,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
- * -----END-LICENCE-----
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 
+#ifndef OMEGA_INCLUDED_CGIPARAM_H
+#define OMEGA_INCLUDED_CGIPARAM_H
+
 typedef multimap<string, string>::const_iterator MCI;
 
 /* decode the query from NAME=VALUE pairs given on the command line */
@@ -36,3 +36,5 @@ extern void decode_post();
 extern void decode_get();
 
 extern multimap<string, string> cgi_params;
+
+#endif // OMEGA_INCLUDED_CGIPARAM_H
