@@ -3,7 +3,8 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2005 James Aylett
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
+ * Copyright 2009 Frank J Bruzzaniti
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -55,6 +56,7 @@
 #include "utils.h"
 #include "values.h"
 #include "xmlparse.h"
+#include "xpsxmlparse.h"
 
 #include "gnu_getopt.h"
 
@@ -157,7 +159,7 @@ get_pdf_metainfo(const string & safefile, string &title, string &keywords)
 		if (pdfinfo[end - 1] == '\r') --end;
 		end -= idx;
 	    }
-	    title = pdfinfo.substr(idx, end);
+	    title.assign(pdfinfo, idx, end);
 	}
 
 	if (strncmp(pdfinfo.c_str(), "Keywords:", 9) == 0) {
@@ -173,7 +175,7 @@ get_pdf_metainfo(const string & safefile, string &title, string &keywords)
 		if (pdfinfo[end - 1] == '\r') --end;
 		end -= idx;
 	    }
-	    keywords = pdfinfo.substr(idx, end);
+	    keywords.assign(pdfinfo, idx, end);
 	}
     } catch (ReadError) {
 	// It's probably best to index the document even if pdfinfo fails.
@@ -211,7 +213,15 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	}
 	MyHtmlParser p;
 	try {
-	    p.parse_html(text);
+	    // Default HTML character set is latin 1, though not specifying one
+	    // is deprecated these days.
+	    p.parse_html(text, "iso-8859-1", false);
+	} catch (const string & newcharset) {
+	    try {
+		p.reset();
+		p.parse_html(text, newcharset, true);
+	    } catch (bool) {
+	    }
 	} catch (bool) {
 	    // MyHtmlParser throws a bool to abandon parsing at </body> or when
 	    // indexing is disallowed
@@ -230,6 +240,7 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    // Currently we assume that text files are UTF-8 unless they have a
 	    // byte-order mark.
 	    dump = file_to_string(file);
+	    md5_string(dump, md5);
 
 	    // Look for Byte-Order Mark (BOM).
 	    if (startswith(dump, "\xfe\xff") || startswith(dump, "\xff\xfe")) {
@@ -248,7 +259,6 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    cout << "can't read \"" << file << "\" - skipping\n";
 	    return;
 	}
-	md5_string(dump, md5);
     } else if (mimetype == "application/pdf") {
 	string safefile = shell_protect(file);
 	string cmd = "pdftotext -enc UTF-8 " + safefile + " -";
@@ -343,6 +353,30 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    cout << "\"" << cmd << "\" failed - skipping\n";
 	    return;
 	}
+    } else if (startswith(mimetype, "application/vnd.openxmlformats-officedocument.")) {
+	const char * args = NULL;
+	string tail(mimetype, 46);
+	if (startswith(tail, "wordprocessingml.")) {
+	    args = " word/document.xml";
+	} else if (startswith(tail, "spreadsheetml.")) {
+	    args = " xl/sharedStrings.xml";
+	} else if (startswith(tail, "presentationml.")) {
+	    args = " ppt/slides/slide*.xml";
+	} else {
+	    // Don't know how to index this type.
+	    cout << "unknown Office 2007 MIME subtype - skipping\n";
+	    return;
+	}
+	string safefile = shell_protect(file);
+	string cmd = "unzip -p " + safefile + args;
+	try {
+	    XmlParser xmlparser;
+	    xmlparser.parse_html(stdout_to_string(cmd));
+	    dump = xmlparser.dump;
+	} catch (ReadError) {
+	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    return;
+	}
     } else if (mimetype == "application/vnd.wordperfect") {
 	// Looking at the source of wpd2html and wpd2text I think both output
 	// utf-8, but it's hard to be sure without sample Unicode .wpd files
@@ -392,7 +426,9 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	string cmd = "unrtf --nopict --html 2>/dev/null " + shell_protect(file);
 	MyHtmlParser p;
 	try {
-	    p.parse_html(stdout_to_string(cmd));
+	    // No point going looking for charset overrides as unrtf doesn't
+	    // produce them.
+	    p.parse_html(stdout_to_string(cmd), "iso-8859-1", true);
 	} catch (ReadError) {
 	    cout << "\"" << cmd << "\" failed - skipping\n";
 	    return;
@@ -445,6 +481,26 @@ index_file(const string &url, const string &mimetype, time_t last_mod, off_t siz
 	    cout << "\"" << cmd << "\" failed - skipping\n";
 	    return;
 	}
+    } else if (mimetype == "application/vnd.ms-xpsdocument") {
+	string safefile = shell_protect(file);
+	string cmd = "unzip -p " + safefile + " Documents/1/Pages/*.fpage";
+	try {
+	    XpsXmlParser xpsparser;
+	    dump = stdout_to_string(cmd);
+	    // Look for Byte-Order Mark (BOM).
+	    if (startswith(dump, "\xfe\xff") || startswith(dump, "\xff\xfe")) {
+		// UTF-16 in big-endian/little-endian order - we just convert
+		// it as "UTF-16" and let the conversion handle the BOM as that
+		// way we avoid the copying overhead of erasing 2 bytes from
+		// the start of dump.
+		convert_to_utf8(dump, "UTF-16");
+	    }
+	    xpsparser.parse_html(dump);
+	    dump = xpsparser.dump;
+	} catch (ReadError) {
+	    cout << "\"" << cmd << "\" failed - skipping\n";
+	    return;
+	}
     } else {
 	// Don't know how to index this type.
 	cout << "unknown MIME type - skipping\n";
@@ -600,6 +656,13 @@ index_directory(size_t depth_limit, const string &dir,
 			if (changed) mt = mime_map.find(ext);
 		    }
 		    if (mt != mime_map.end()) {
+			if (mt->second.empty()) {
+			    cout << "Skipping file, required filter not "
+				    "installed: \"" << file << "\""
+				 << endl;
+			    continue;
+			}
+
 			// Only check the file size if we recognise the
 			// extension to avoid a call to stat()/lstat() for
 			// files we can't handle when readdir() tells us the
@@ -622,7 +685,7 @@ index_directory(size_t depth_limit, const string &dir,
 			    cout << "Filter for \"" << mimetype
 				 << "\" not installed - ignoring extension \""
 				 << ext << "\"" << endl;
-			    mime_map.erase(mt);
+			    mt->second = string();
 			}
 		    } else {
 			cout << "Unknown extension: \"" << file
@@ -717,6 +780,15 @@ main(int argc, char **argv)
     mime_map["sxw"] = "application/vnd.sun.xml.writer";
     mime_map["sxg"] = "application/vnd.sun.xml.writer.global";
     mime_map["stw"] = "application/vnd.sun.xml.writer.template";
+    // MS Office 2007 formats:
+    mime_map["docx"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.document"; // Word 2007
+    mime_map["dotx"] = "application/vnd.openxmlformats-officedocument.wordprocessingml.template"; // Word 2007 template
+    mime_map["xlsx"] = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"; // Excel 2007
+    mime_map["xltx"] = "application/vnd.openxmlformats-officedocument.spreadsheetml.template"; // Excel 2007 template
+    mime_map["pptx"] = "application/vnd.openxmlformats-officedocument.presentationml.presentation"; // PowerPoint 2007 presentation
+    mime_map["ppsx"] = "application/vnd.openxmlformats-officedocument.presentationml.slideshow"; // PowerPoint 2007 slideshow
+    mime_map["potx"] = "application/vnd.openxmlformats-officedocument.presentationml.template"; // PowerPoint 2007 template
+    mime_map["xps"] = "application/vnd.ms-xpsdocument";
     // Some other word processor formats:
     mime_map["doc"] = "application/msword";
     mime_map["dot"] = "application/msword"; // Word template
@@ -742,7 +814,7 @@ main(int argc, char **argv)
     mime_map["djv"] = "image/vnd.djvu";
     mime_map["djvu"] = "image/vnd.djvu";
 
-    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:lpf", longopts, NULL)) != -1) {
+    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:l:s:pf", longopts, NULL)) != -1) {
 	switch (getopt_ret) {
 	case 'h': {
 	    cout << PROG_NAME" - "PROG_DESC"\n\n"
@@ -764,13 +836,6 @@ main(int argc, char **argv)
 	}
 	case 'v':
 	    print_package_info(PROG_NAME);
-	    cout << "\n"
-		 << "Copyright (c) 1999,2000,2001 BrightStation PLC.\n"
-		 << "Copyright (c) 2001,2005 James Aylett\n"
-		 << "Copyright (c) 2001,2002 Ananova Ltd\n"
-		 << "Copyright (c) 2002,2003,2004,2005,2006 Olly Betts\n\n"
-		 << "This is free software, and may be redistributed under\n"
-		 << "the terms of the GNU Public License." << endl;
 	    return 0;
 	case 'd': // how shall we handle duplicate documents?
 	    switch (optarg[0]) {
