@@ -4,7 +4,7 @@
  * Copyright 2001 James Aylett
  * Copyright 2001,2002 Ananova Ltd
  * Copyright 2002 Intercede 1749 Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009 Olly Betts
  * Copyright 2008 Thomas Viehmann
  *
  * This program is free software; you can redistribute it and/or
@@ -603,7 +603,7 @@ html_highlight(const string &s, const string &list,
 		Xapian::Unicode::append_utf8(term, *j);
 	    }
 	    if (term.length() < 2 || (j != s_end && is_wordchar(*j))) {
-		term = "";
+		term.resize(0);
 	    }
 	    term_end = j;
 	}
@@ -944,6 +944,23 @@ T(weight,	   0, 0, N, 0), // weight of the current hit
 
 static vector<string> macros;
 
+// Call write() repeatedly until all data is written or we get a
+// non-recoverable error.
+static ssize_t
+write_all(int fd, const char * buf, size_t count)
+{
+    while (count) {
+	ssize_t r = write(fd, buf, count);
+	if (rare(r < 0)) {
+	    if (errno == EINTR) continue;
+	    return r;
+	}
+	buf += r;
+	count -= r;
+    }
+    return 0;
+}
+
 static string
 eval(const string &fmt, const vector<string> &param)
 {
@@ -1089,7 +1106,7 @@ eval(const string &fmt, const vector<string> &param)
 		for (vector<string>::const_iterator i = args.begin();
 		     i != args.end(); i++) {
 		    if (eval(*i, param).empty()) {
-			value = "";
+			value.resize(0);
 			break;
 		    }
 	        }
@@ -1264,8 +1281,8 @@ eval(const string &fmt, const vector<string> &param)
 		try {
 		    value = int_to_string(mset.get_termfreq(args[0]));
 		} catch (const Xapian::InvalidOperationError&) {
-		    // An mset will raise this error if it's empty and not associated with
-		    // a search.
+		    // An MSet will raise this error if it's empty and not
+		    // associated with a search.
 		    value = int_to_string(db.get_termfreq(args[0]));
 		}
 		break;
@@ -1449,7 +1466,7 @@ eval(const string &fmt, const vector<string> &param)
 		}
 		line = eval(line, noargs);
 		line += '\n';
-		write(fd, line.data(), line.length());
+		(void)write_all(fd, line.data(), line.length());
 		close(fd);
 		break;
 	    }
@@ -2297,8 +2314,8 @@ OmegaExpandDecider::operator()(const string & term) const
     if (exclude_stems.find(stem) != exclude_stems.end())
 	return false;
 
-    // Ignore terms that only occurs once (hapaxes) since they aren't
-    // useful for finding related documents - they only occurs in a
+    // Ignore terms that only occur once (hapaxes) since they aren't
+    // useful for finding related documents - they only occur in a
     // document that's already been marked as relevant.
     // FIXME: add an expand option to ignore terms where
     // termfreq == rtermfreq.
