@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001 Sam Liddicott
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2009 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -34,6 +34,7 @@
 #include <vector>
 #include <cstring>
 
+#include <cstdlib>
 #include "safeerrno.h"
 #include <stdio.h>
 #include <time.h>
@@ -95,7 +96,7 @@ prefix_needs_colon(const string & prefix, unsigned ch)
 const char * action_names[] = {
     "bad", "new",
     "boolean", "date", "field", "hash", "index", "indexnopos", "load", "lower",
-    "truncate", "unhtml", "unique", "value", "weight"
+    "truncate", "unhtml", "unique", "value", "valuenumeric", "weight"
 };
 
 // For debugging:
@@ -106,14 +107,15 @@ public:
     typedef enum {
 	BAD, NEW,
 	BOOLEAN, DATE, FIELD, HASH, INDEX, INDEXNOPOS, LOAD, LOWER,
-	TRUNCATE, UNHTML, UNIQUE, VALUE, WEIGHT
+	TRUNCATE, UNHTML, UNIQUE, VALUE, VALUENUMERIC, WEIGHT
     } type;
 private:
     type action;
     int num_arg;
     string string_arg;
 public:
-    Action(type action_, string arg = "") : action(action_), string_arg(arg) {
+    Action(type action_) : action(action_), num_arg(0) { }
+    Action(type action_, string arg) : action(action_), string_arg(arg) {
 	num_arg = atoi(string_arg.c_str());
     }
     Action(type action_, string arg, int num)
@@ -253,6 +255,10 @@ parse_index_script(const string &filename)
 			    code = Action::VALUE;
 			    arg = YES;
 			    takes_integer_argument = true;
+			} else if (action == "valuenumeric") {
+			    code = Action::VALUENUMERIC;
+			    arg = YES;
+			    takes_integer_argument = true;
 			}
 			break;
 		    case 'w':
@@ -537,7 +543,12 @@ index_file(const char *fname, istream &stream,
 		    case Action::UNHTML: {
 			MyHtmlParser p;
 			try {
-			    p.parse_html(value);
+			    // Default HTML character set is latin 1, though
+			    // not specifying one is deprecated these days.
+			    p.parse_html(value, "iso-8859-1", false);
+			} catch (const string & newcharset) {
+			    p.reset();
+			    p.parse_html(value, newcharset, true);
 			} catch (bool) {
 			    // MyHtmlParser throws a bool to abandon parsing at
 			    // </body> or when indexing is disallowed
@@ -582,7 +593,6 @@ again:
 			    cout << "Caught exception in UNIQUE!" << endl;
 			    cout << "E: " << e.get_msg() << endl;
 			    database.flush();
-			    database.reopen();
 			    goto again;
 			}
 			break;
@@ -591,6 +601,19 @@ again:
 			if (!value.empty())
 			    doc.add_value(i->get_num_arg(), value);
 			break;
+		    case Action::VALUENUMERIC: {
+			if (value.empty()) break;
+			char * end;
+			double dbl = strtod(value.c_str(), &end);
+			if (*end) {
+			    cout << fname << ':' << line_no << ": Warning: "
+				    "Trailing characters in VALUENUMERIC: '"
+				 << value << "'" << endl;
+			}
+			doc.add_value(i->get_num_arg(),
+				      Xapian::sortable_serialise(dbl));
+			break;
+		    }
 		    case Action::DATE: {
 			const string & type = i->get_string_arg();
 			string yyyymmdd;
